<?php
/**
 * iPay payment plugin
 * 
 * @author Nicholas Kimuli
 * @version 1.0.2
 * @package VirtueMart
 * @subpackage payment
 *
 * Copyright (c) 2017. All rights reserved iPay - www.ipayafrica.com
 *
 * This program is free software. You are allowed to use the software but NOT allowed to modify the software. 
 * It is also not legal to do any changes to the software and distribute it in your own name / brand. 
 *
 */

defined('_JEXEC') or die('Direct Access to ' . basename(__FILE__ ) . ' is not allowed.');
	
if (!class_exists('vmPSPlugin'))
    require(JPATH_VM_PLUGINS . DS . 'vmpsplugin.php');

class plgVmPaymentIPay extends vmPSPlugin {

    // instance of class
    public static $_this = false;

    function __construct(& $subject, $config) {
		//if (self::$_this)
		 //   return self::$_this;
		parent::__construct($subject, $config);

	    $this->_loggable = true;
	    $this->tableFields = array_keys($this->getTableSQLFields());
		$this->_tablepkey = 'id';
		$this->_tableId = 'id';
		$varsToPush = $this->getVarsToPush();

	    $this->setConfigParameterable($this->_configTableFieldName, $varsToPush);
	   // self::$_this = $this;

    }

    /**
     * Create the table for this plugin if it does not yet exist.
     * @author Valérie Isaksen
     */
    protected function getVmPluginCreateTableSQL() {
		return $this->createTableSQL('Payment Ipay Table');
    }

    /**
     * Fields to create the payment table
     * @return string SQL Fileds
     */
    function getTableSQLFields() {
		$SQLfields = [
			'id' => 'int(11) UNSIGNED NOT NULL AUTO_INCREMENT',
			'virtuemart_order_id' => 'int(1) UNSIGNED',
			'order_number' => 'char(64)',
			'virtuemart_paymentmethod_id' => 'mediumint(1) UNSIGNED',
			'payment_name' => 'varchar(5000)',
			'payment_order_total' => 'decimal(15,5) NOT NULL',
			'payment_currency' => 'smallint(1)',
			'email_currency' => 'smallint(1)',
			'cost_per_transaction' => 'decimal(10,2)',
			'cost_percent_total' => 'decimal(10,2)',
			'tax_id' => 'smallint(1)',
			'paypal_custom' => 'varchar(255)',
			'paypal_method' => 'varchar(200)' // Add extra iPay fields if need be
		];

		return $SQLfields;
    }

    /**
     * Processing the payment on checkout
	 *
     */
    function plgVmConfirmedOrder($cart, $order) {

		if (!($method = $this->getVmPluginMethod($order['details']['BT']->virtuemart_paymentmethod_id))) {
		    return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
		    return false;
		}

		// $params = new JParameter($payment->payment_params);
		$lang = JFactory::getLanguage();
		$filename = 'com_virtuemart';
		$lang->load($filename, JPATH_ADMINISTRATOR);
		
		if (!class_exists('VirtueMartModelOrders')) {
		    require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
		}
		if (!class_exists('VirtueMartModelCurrency')) {
			require(VMPATH_ADMIN . DS . 'models' . DS . 'currency.php');
		}

		// $method = $this->getPluginMethod(vRequest::getInt('virtuemart_paymentmethod_id'));

		// Getting variables from admin section and passing them to iPay API
		// iPay v.3
		$this->getPaymentCurrency($method);

		$q = 'SELECT `currency_code_3` FROM `#__virtuemart_currencies` WHERE `virtuemart_currency_id`="'.$method->payment_currency.'" ';
		$db = &JFactory::getDBO();
		$db->setQuery($q);
		$currency_code_from_db = $db ->loadObject();
		$currency_code = $currency_code_from_db->currency_code_3;

	    $this->_virtuemart_paymentmethod_id = $order['details']['BT']->virtuemart_paymentmethod_id;
		$live = $method->live;
		$mpesa = $method->mpesa;
		$equity = $method->equity;
		$mobilebanking = $method->mobilebanking;
		$airtel = $method->airtel;
		$creditcard = $method->creditcard;
		$debitcard = $method->debitcard;
		$mkoporahisi = $method->mkoporahisi;
		$saida = $method->saida;
		$vid = $method->vendorid;
		$order_id = $order['details']['BT']->order_number;
		$invoice = $order_id;
		$total = number_format(ceil($order['details']['BT']->order_total), 2, '.', '');
		$phone = $order['details']['BT']->phone_1;
		$phone2 = $order['details']['BT']->phone_2;
		$email = $order['details']['BT']->email;
		$vendor_ref = $method->vendorid;
		$currency_code_3 = $currency_code;
	   
		$curr = $currency_code_3;
		$p1 = $this->_virtuemart_paymentmethod_id;
		$p2 = $curr;
		$p3 = "";
		$p4 = "";

		// Separate callback file
		// $cbk = JURI::base().'plugins/vmpayment/ipay/helpers/callback.php';
		
		// Callback function in this file
		$cbk = JURI::base();
		$cbk .= 'index.php?callback=1&option=com_virtuemart&view=pluginresponse&task=pluginresponsereceived&pm='; 
		$cbk .= $this->_virtuemart_paymentmethod_id;

		$cst = 1;
		$crl = 0; // Use 2 for a json data stream

		$datastring = $live.$order_id.$invoice.$total.$phone.$email.$vid.$curr.$p1.$p2.$p3.$p4.$cbk.$cst.$crl;

		// Get security key from admin settings
		$hashkey = $method->hashkey;
		$datastring;
		$hashid = hash_hmac('sha1', $datastring, $hashkey);

		// Encode callback url
		$cbk = urlencode($cbk);

		$url = 'https://payments.ipayafrica.com/v3/ke?live='.$live.'&mpesa='.$mpesa.'&airtel='.$airtel.'&equity='.$equity.'&mobilebanking='.$mobilebanking.'&debitcard='.$debitcard.'&creditcard='.$creditcard.'&mkoporahisi='.$mkoporahisi.'&saida='.$saida.'&oid='.$order_id.'&inv='.$invoice.'&ttl='.$total.'&tel='.$phone.'&eml='.$email.'&vid='.$vid.'&p1='.$p1.'&p2='.$p2.'&p3='.$p3.'&p4='.$p4.'&crl='.$crl.'&cbk='.$cbk.'&cst='.$cst.'&curr='.$curr.'&hsh='.$hashid;

		if ($method->paymentprocess) { // if true / 1
			// 1. Show iframe
			$code='<iframe src="'.$url.'" height="1000" width="700" /></iframe>';
			
		} else { 
			// 2. Redirect to iPay page
			$code='<script>window.location = "'.$url.'";</script>';

		}
	
		$paymentCurrency = CurrencyDisplay::getInstance($method->payment_currency);
		$totalInPaymentCurrency = round($paymentCurrency->convertCurrencyTo($method->payment_currency, $order['details']['BT']->order_total, false), 2);
		$cd = CurrencyDisplay::getInstance($cart->pricesCurrency);
	
		$this->_virtuemart_paymentmethod_id = $order['details']['BT']->virtuemart_paymentmethod_id;
		$dbValues['payment_name'] = $this->renderPluginName($method);
		$dbValues['order_number'] = $order['details']['BT']->order_number;
		$dbValues['virtuemart_paymentmethod_id'] = $this->_virtuemart_paymentmethod_id;
		$dbValues['cost_per_transaction'] = $method->cost_per_transaction;
		$dbValues['cost_percent_total'] = $method->cost_percent_total;
		$dbValues['payment_currency'] = $curr;
		$dbValues['payment_order_total'] = $totalInPaymentCurrency;
		$dbValues['tax_id'] = $method->tax_id;
		$this->storePSPluginInternalData($dbValues);
	
		// return $this->processConfirmedOrderPaymentResponse(true, $cart, $order, $code); // return true, empty cart, send order
		$cart->_confirmDone = false;
		$cart->_dataValidated = false;
		$cart->setCartIntoSession();
		vRequest::setVar('html', $code);
    }

    /**
     * Display stored payment data for an order
     *
     */
    function plgVmOnShowOrderBEPayment($virtuemart_order_id, $virtuemart_payment_id) {
		if (!$this->selectedThisByMethodId($virtuemart_payment_id)) {
	   	return null; // Another method was selected, do nothing
	 	}

		$db = JFactory::getDBO();
		$q = 'SELECT * FROM `' . $this->_tablename . '` '
			. 'WHERE `virtuemart_order_id` = ' . $virtuemart_order_id;
		$db->setQuery($q);
		if (!($paymentTable = $db->loadObject())) {
		    vmWarn(500, $q . " " . $db->getErrorMsg());
		    return '';
		}
		$this->getPaymentCurrency($paymentTable);
	
		$html = '<table class="adminlist">' . "\n";
		$html .=$this->getHtmlHeaderBE();
		$html .= $this->getHtmlRowBE('STANDARD_PAYMENT_NAME', $paymentTable->payment_name);
		$html .= $this->getHtmlRowBE('STANDARD_PAYMENT_TOTAL_CURRENCY', $paymentTable->payment_order_total.' '.$paymentTable->payment_currency);
		$html .= '</table>' . "\n";
		return $html;
    }

    function getCosts(VirtueMartCart $cart, $method, $cart_prices) {
		if (preg_match('/%$/', $method->cost_percent_total)) {
		    $cost_percent_total = substr($method->cost_percent_total, 0, -1);
		} else {
		    $cost_percent_total = $method->cost_percent_total;
		}
		return ($method->cost_per_transaction + ($cart_prices['salesPrice'] * $cost_percent_total * 0.01));
    }

    /**
     * Check if the payment conditions are fulfilled for this payment method
     * @author: Valerie Isaksen
     *
     * @param $cart_prices: cart prices
     * @param $payment
     * @return true: if the conditions are fulfilled, false otherwise
     *
     */
    protected function checkConditions($cart, $method, $cart_prices) {

		// $params = new JParameter($payment->payment_params);
		$address = (($cart->ST == 0) ? $cart->BT : $cart->ST);
	
		$amount = $cart_prices['salesPrice'];
		$amount_cond = ($amount >= $method->min_amount AND $amount <= $method->max_amount
			OR
			($method->min_amount <= $amount AND ($method->max_amount == 0) ));
		if (!$amount_cond) {
		    return false;
		}
		$countries = array();
		if (!empty($method->countries)) {
		    if (!is_array($method->countries)) {
			$countries[0] = $method->countries;
		    } else {
			$countries = $method->countries;
		    }
		}
	
		// probably did not gave his BT:ST address
		if (!is_array($address)) {
		    $address = array();
		    $address['virtuemart_country_id'] = 0;
		}
	
		if (!isset($address['virtuemart_country_id']))
		    $address['virtuemart_country_id'] = 0;
		if (count($countries) == 0 || in_array($address['virtuemart_country_id'], $countries) || count($countries) == 0) {
		    return true;
		}
	
		return false;
    }

    /*
     * We must reimplement this triggers for joomla 1.7
     */

    /**
     * Create the table for this plugin if it does not yet exist.
     * This functions checks if the called plugin is active one.
     * When yes it is calling the standard method to create the tables
     * @author Valérie Isaksen
     *
     */
    function plgVmOnStoreInstallPaymentPluginTable($jplugin_id) {
		return $this->onStoreInstallPluginTable($jplugin_id);
    }

    /**
     * This event is fired after the payment method has been selected. It can be used to store
     * additional payment info in the cart.
     *
     * @author Max Milbers
     * @author Valérie isaksen
     *
     * @param VirtueMartCart $cart: the actual cart
     * @return null if the payment was not selected, true if the data is valid, error message if the data is not vlaid
     *
     */
    public function plgVmOnSelectCheckPayment(VirtueMartCart $cart) {
		return $this->OnSelectCheck($cart);
    }

    /**
     * plgVmDisplayListFEPayment
     * This event is fired to display the pluginmethods in the cart (edit shipment/payment) for exampel
     *
     * @param object $cart Cart object
     * @param integer $selected ID of the method selected
     * @return boolean True on succes, false on failures, null when this plugin was not selected.
     * On errors, JError::raiseWarning (or JError::raiseError) must be used to set a message.
     *
     * @author Valerie Isaksen
     * @author Max Milbers
     */
    public function plgVmDisplayListFEPayment(VirtueMartCart $cart, $selected = 0, &$htmlIn) {
		return $this->displayListFE($cart, $selected, $htmlIn);
    }

    /*
     * plgVmonSelectedCalculatePricePayment
     * Calculate the price (value, tax_id) of the selected method
     * It is called by the calculator
     * This function does NOT to be reimplemented. If not reimplemented, then the default values from this function are taken.
     * @author Valerie Isaksen
     * @cart: VirtueMartCart the current cart
     * @cart_prices: array the new cart prices
     * @return null if the method was not selected, false if the shiiping rate is not valid any more, true otherwise
     *
     *
     */
    public function plgVmonSelectedCalculatePricePayment(VirtueMartCart $cart, array &$cart_prices, &$cart_prices_name) {
		return $this->onSelectedCalculatePrice($cart, $cart_prices, $cart_prices_name);
    }

    function plgVmgetPaymentCurrency($virtuemart_paymentmethod_id, &$paymentCurrencyId) {

		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
		    return null; // Another method was selected, do nothing
		}
		if (!$this->selectedThisElement($method->payment_element)) {
		    return false;
		}

		$this->getPaymentCurrency($method);
	
		$paymentCurrencyId = $method->payment_currency;

	}

    /**
     * plgVmOnCheckAutomaticSelectedPayment
     * Checks how many plugins are available. If only one, the user will not have the choice. Enter edit_xxx page
     * The plugin must check first if it is the correct type
     * @author Valerie Isaksen
     * @param VirtueMartCart cart: the cart object
     * @return null if no plugin was found, 0 if more then one plugin was found,  virtuemart_xxx_id if only one plugin is found
     *
     */
    function plgVmOnCheckAutomaticSelectedPayment(VirtueMartCart $cart, array $cart_prices = array()) {
		return $this->onCheckAutomaticSelected($cart, $cart_prices);
    }

    /**
     * This method is fired when showing the order details in the frontend.
     * It displays the method-specific data.
     *
     * @param integer $order_id The order ID
     * @return mixed Null for methods that aren't active, text (HTML) otherwise
     * @author Max Milbers
     * @author Valerie Isaksen
     */
    protected function plgVmOnShowOrderFEPayment($virtuemart_order_id, $virtuemart_paymentmethod_id, &$payment_name) {
		$this->onShowOrderFE($virtuemart_order_id, $virtuemart_paymentmethod_id, $payment_name);
    }

    /**
     * This method is fired when showing when priting an Order
     * It displays the the payment method-specific data.
     *
     * @param integer $_virtuemart_order_id The order ID
     * @param integer $method_id  method used for this order
     * @return mixed Null when for payment methods that were not selected, text (HTML) otherwise
     * @author Valerie Isaksen
     */
    function plgVmonShowOrderPrintPayment($order_number, $method_id) {
		return $this->onShowOrderPrint($order_number, $method_id);
    }

    function plgVmDeclarePluginParamsPayment($name, $id, &$data) {
		return $this->declarePluginParams('payment', $name, $id, $data);
    }

    function plgVmSetOnTablePluginParamsPayment($name, $id, &$table) {
		return $this->setOnTablePluginParams($name, $id, $table);
    }
    
     /**
     * plgVmOnPaymentResponseReceived
     * This event is fired when the  method returns to the shop after the transaction
     *
     *  the method itself should send in the URL the parameters needed
     * NOTE for Plugin developers:
     *  If the plugin is NOT actually executed (not the selected payment method), this method must return NULL
     *
     * @param int $virtuemart_order_id : should return the virtuemart_order_id
     * @param text $html: the html to display
     * @return mixed Null when this method was not selected, otherwise the true or false
     *
     * @author Valerie Isaksen
     *
     **/
    function plgVmOnPaymentResponseReceived(&$virtuemart_order_id, &$html, $order) {
		if (!class_exists('VirtueMartModelOrders')) {
			require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
		}

		if (!class_exists('VirtueMartCart')) {
			require(VMPATH_SITE . DS . 'helpers' . DS . 'cart.php');
		}
		
		// Get returned variables from iPay
		$payment_data = JRequest::get('get');
					
		// The payment itself should send the parameter needed.
		$virtuemart_paymentmethod_id = JRequest::getInt('p1', 0);

		// Get admin settings
		if (!($method = $this->getVmPluginMethod($virtuemart_paymentmethod_id))) {
		    return null; // Another method was selected, do nothing
		}

		if (!$this->selectedThisElement($method->payment_element)) {
			return false;
		}

		vmdebug('plgVmOnPaymentResponseReceived', $payment_data);
		
		// Getting the order
		$order_number = $payment_data['id'];
		$virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number);

		$this->logInfo('plgVmOnPaymentNotification: virtuemart_order_id  found ' . $virtuemart_order_id, 'message');

		// Confirming if the payment is genuine and successfull
		$val = $method->vendorid;

		$val1 = $order_number; 
		$val2 = $payment_data['ivm'];
		$val3 = $payment_data['qwh'];
		$val4 = $payment_data['afd'];
		$val5 = $payment_data['poi'];
		$val6 = $payment_data['uyt'];
		$val7 = $payment_data['ifd'];

		$ipnurl= "https://www.ipayafrica.com/ipn/?vendor=".$val."&id=".$val1."&ivm=".
		$val2."&qwh=".$val3."&afd=".$val4."&poi=".$val5."&uyt=".$val6."&ifd=".$val7; 

		$ch = curl_init($ipnurl);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HEADER, 0);
		$result = curl_exec($ch);
		curl_close($ch);
		
		$db = JFactory::getDBO();
		$session = JFactory::getSession();
		$return_context = $session->getId();

		if (strcmp($result, 'aei7p7yrx4ae34') == 0): //for success..
			$new_status = 'C';
			$msg = 'Your transaction was successful.';
			$msg = '<center><div class="alert alert-success ipay" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
		elseif(strcmp($result, 'fe2707etr5s4wq') == 0):
			$new_status = 'C';
			$msg = 'Your transaction failed. Please contact the store owner';
			$msg = '<center><div class="alert alert-danger" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
		elseif(strcmp($result, 'bdi6p2yy76etrs') == 0): //pending..
			$new_status = 'P';
			$msg = 'Your transaction is pending. Please try again later';
			$msg = '<center><div class="alert alert-warning" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
		elseif(strcmp($result, 'eq3i7p5yt7645e') == 0): //more.. 		
			$new_status = 'C';
			$msg = 'Your transaction was successful.However,the amount you sent was more than required. Please notify your merchant.';
			$msg = '<center><div class="alert alert-warning" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';			
		elseif(strcmp($result, 'cr5i3pgy9867e1') == 0): //already used..
			$new_status = 'P';
			$msg = 'You are trying to use a code that has already been used. Please use a valid code';
			$msg = '<center><div class="alert alert-warning" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
		else:
			$new_status = 'P';
			$msg = 'You did not pay the correct amount. Please contact support';
			$msg = '<center><div class="alert alert-warning" role="alert">
    	<center><strong>'.$msg.'</center></strong></div></center>';
		endif;
		
		$this->logInfo('plgVmOnPaymentNotification return new_status' . $new_status, 'message');
		
		$virtuemart_order_id = VirtueMartModelOrders::getOrderIdByOrderNumber($order_number);
		
		if ($virtuemart_order_id) {
			// send the email only if payment has been accepted
			if (!class_exists('VirtueMartModelOrders'))
			require( JPATH_VM_ADMINISTRATOR . DS . 'models' . DS . 'orders.php' );
			$modelOrder = new VirtueMartModelOrders();
			$orders[$virtuemart_order_id]['order_status'] = $new_status;
			$orders[$virtuemart_order_id]['virtuemart_order_id'] = $virtuemart_order_id;
			$orders[$virtuemart_order_id]['customer_notified'] = 0;
			
			// Take the session directly from the DB
			$modelOrder->updateOrderStatus($orders); 
			
			$order_data = $this->getDataByOrderId($virtuemart_order_id);
			$payment_name = $this->renderPluginName($method);
			$payment_currency = $payment_data['p2'];
			$html = $this->_getPaymentResponseHtml($payment_data, $payment_name, $order_data, $payment_currency, $msg);
			// echo $html;
			// remove vmcart
		}
		
		//We delete the old stuff
		// get the correct cart / session
		$cart = VirtueMartCart::getCart();
		$cart->emptyCart();
		return true;
    }
    
    function emptyCart($session_name) {

    }
    
    function _getPaymentResponseHtml($paypal_data, $payment_name, $order_data, $payment_currency, $msg) {

		$html = '<table>' . "\n";
		
			$html .= '<thead><th colspan="2"><b>' . $msg ."</b></th></thead>";
			$html .= '<tr><td>Payment Method</td><td>' . $payment_name ."</td></tr>";
			$html .= '<tr><td>Order Number</td><td>' . $paypal_data['id'] ."</td></tr>";
			$html .= '<tr><td>Amount</td><td>' . $payment_currency.'  '.number_format(round($order_data->payment_order_total), 2) ."</td></tr>";

		$html .= '</table>' . "\n";

		return $html;
    }
    
    function plgVmDeclarePluginParamsPaymentVM3( &$data) {
		return $this->declarePluginParams('payment', $data);
	}
}

// No closing tag
